@@ -1,7 +1,7 @@
 /* 
    Project: DataBasin
 
-   Copyright (C) 2008-2014 Free Software Foundation
+   Copyright (C) 2008-2016 Free Software Foundation
 
    Author: Riccardo Mottola
 
@@ -44,28 +44,12 @@
 
 @implementation AppController
 
-+ (void)initialize
-{
-  NSMutableDictionary *defaults = [NSMutableDictionary dictionary];
-
-  /*
-   * Register your app's defaults here by adding objects to the
-   * dictionary, eg
-   *
-   * [defaults setObject:anObject forKey:keyForThatObject];
-   *
-   */
-  
-  [[NSUserDefaults standardUserDefaults] registerDefaults:defaults];
-  [[NSUserDefaults standardUserDefaults] synchronize];
-}
-
 - (id)init
 {
   if ((self = [super init]))
     {
       NSUserDefaults *defaults;
-
+      
       defaults = [NSUserDefaults standardUserDefaults];
 
       logger = [[DBLogger alloc] init];
@@ -81,7 +65,9 @@
       /* if none found, set a reasonable default for the upload and insert batch size */
       if (![defaults objectForKey:@"UpBatchSize"])
 	[defaults setObject:[NSNumber numberWithInt:100] forKey:@"UpBatchSize"];
-
+      /* if none found, set a reasonable default for the query batch size */
+      if (![defaults objectForKey:@"DownBatchSize"])
+	[defaults setObject:[NSNumber numberWithInt:500] forKey:@"DownBatchSize"];
     }
   return self;
 }
@@ -134,20 +120,30 @@
       obj = [NSNumber numberWithInt: LogStandard];
       [defaults setObject:obj forKey: @"LogLevel"];
     }
-
   [logger setLogLevel: [obj intValue]];
-  
-  size = [defaults integerForKey:@"UpBatchSize"];
-  if (size > 0)
-    {
-      [db setUpBatchSize:size];
+
+  obj = [defaults objectForKey: @"StringEncoding"];
+  if (obj == nil)
+	{
+	  obj = [NSNumber numberWithInt: NSUTF8StringEncoding];
+	  [defaults setObject:obj forKey: @"StringEncoding"];
     }
+	
+  size = [defaults integerForKey:@"UpBatchSize"];
+  if (size == 0)
+    {
+      size = 10;
+      [defaults setInteger:size forKey:@"UpBatchSize"];
+    }
+  [db setUpBatchSize:size];
 
   size = [defaults integerForKey:@"DownBatchSize"];
-  if (size > 0)
+  if (size == 0)
     {
-      [db setDownBatchSize:size];
+      size = 100;
+      [defaults setInteger:size forKey:@"DownBatchSize"];
     }
+  [db setDownBatchSize:size];
 
   // FIXME here we should set the defaults of the CSV reader/writers
 }
@@ -163,12 +159,16 @@
     }
   
   [preferences showPrefPanel:sender];
-
-  /* Apply defaults */
-  defaults = [NSUserDefaults standardUserDefaults];
-  [logger setLogLevel: [defaults integerForKey: @"StringEncoding"]];
 }
 
+- (void)showException:(NSException *)e
+{
+  NSLog(@"Exception: %@ - %@", e, [e reason]);
+  [faultTextView setString:[e reason]];
+  [faultPanel makeKeyAndOrderFront:self];
+}
+
+
 
 /* SESSION INSPECTOR */
 
@@ -228,17 +228,22 @@
   NSURL    *url;
   NSDictionary *uInfo;
   NSMutableDictionary *loginSet;
+  NSUserDefaults *defaults;
   
   userName = [fieldUserName stringValue];
   password = [fieldPassword stringValue];
   token = [fieldToken stringValue];
 
+  defaults = [NSUserDefaults standardUserDefaults];
+    
   /* if present, we append the security token to the password */
   if (token != nil)
     password = [password stringByAppendingString:token];
     
   db = [[DBSoap alloc] init];
   [db setLogger: logger];
+  [db setUpBatchSize:[[defaults objectForKey:@"UpBatchSize"] intValue]];
+  [db setDownBatchSize:[[defaults objectForKey:@"DownBatchSize"] intValue]];
   dbCsv = [[DBSoapCSV alloc] init];
   [dbCsv setDBSoap:db];
   
@@ -277,8 +282,7 @@
   [loginStatus setImage:[NSImage imageNamed:@"butt_red_16.tif"]];
     if ([[localException name] hasPrefix:@"DB"])
       {
-        [faultTextView setString:[localException reason]];
-        [faultPanel makeKeyAndOrderFront:nil];
+        [self performSelectorOnMainThread:@selector(showException:) withObject:localException waitUntilDone:YES];
 	return;
       }
     else
@@ -365,17 +369,24 @@
     }
 }
 
-- (IBAction)executeSelect:(id)sender
+- (void)resetSelectUI:(id)arg
+{
+  [buttonSelectExec setEnabled:YES];
+  [buttonSelectStop setEnabled:NO];
+}
+
+- (void)performSelect:(id)arg
 {
   NSString       *statement;
   NSString       *filePath;
   NSFileHandle   *fileHandle;
   NSFileManager  *fileManager;
   DBCSVWriter    *csvWriter;
-  DBProgress     *progress;
   NSString       *str;
   NSUserDefaults *defaults;
+  NSAutoreleasePool *arp;
   
+  arp = [NSAutoreleasePool new];
   defaults = [NSUserDefaults standardUserDefaults];
   statement = [fieldQuerySelect string];
   filePath = [fieldFileSelect stringValue];
@@ -384,6 +395,7 @@
   if ([fileManager createFileAtPath:filePath contents:nil attributes:nil] == NO)
     {
       NSRunAlertPanel(@"Attention", @"Could not create File.", @"Ok", nil, nil);
+      [self performSelectorOnMainThread:@selector(resetSelectUI:) withObject:self waitUntilDone:NO];
       return;
     }  
 
@@ -391,16 +403,19 @@
   if (fileHandle == nil)
     {
       NSRunAlertPanel(@"Attention", @"Cannot create File.", @"Ok", nil, nil);
+      [self performSelectorOnMainThread:@selector(resetSelectUI:) withObject:self waitUntilDone:NO];
+      return;
     }
-  
-  progress = [[DBProgress alloc] init];
-  [progress setProgressIndicator: progIndSelect];
-  [progress setRemainingTimeField: fieldRTSelect];
-  [progress setLogger:logger];
-  [progress reset];
+
+  selectProgress = [[DBProgress alloc] init];
+  [selectProgress setProgressIndicator: progIndSelect];
+  [selectProgress setRemainingTimeField: fieldRTSelect];
+  [selectProgress setLogger:logger];
+  [selectProgress reset];
   csvWriter = [[DBCSVWriter alloc] initWithHandle:fileHandle];
   [csvWriter setLogger:logger];
   [csvWriter setWriteFieldsOrdered:([orderedWritingSelect state] == NSOnState)];
+  [csvWriter setLineBreakHandling:[defaults integerForKey:CSVWriteLineBreakHandling]];
   str = [defaults stringForKey:@"CSVWriteQualifier"];
   if (str)
     [csvWriter setQualifier:str];
@@ -409,22 +424,44 @@
     [csvWriter setSeparator:str];
   
   NS_DURING
-    [dbCsv query :statement queryAll:([queryAllSelect state] == NSOnState) toWriter:csvWriter progressMonitor:progress];
+    [dbCsv query :statement queryAll:([queryAllSelect state] == NSOnState) toWriter:csvWriter progressMonitor:selectProgress];
   NS_HANDLER
     if ([[localException name] hasPrefix:@"DB"])
       {
-        [faultTextView setString:[localException reason]];
-        [faultPanel makeKeyAndOrderFront:nil];
+        [self performSelectorOnMainThread:@selector(showException:) withObject:localException waitUntilDone:YES];
       }
   NS_ENDHANDLER
   [csvWriter release];
   [fileHandle closeFile];
-  [progress release];
+  [selectProgress release];
+  selectProgress = nil;
+  [self performSelectorOnMainThread:@selector(resetSelectUI:) withObject:self waitUntilDone:NO];
+  [arp drain];
+}
+
+- (IBAction)executeSelect:(id)sender
+{
+  [buttonSelectExec setEnabled:NO];
+  [buttonSelectStop setEnabled:YES];
+  [NSThread detachNewThreadSelector:@selector(performSelect:) toTarget:self withObject:nil];
+}
+
+
+- (IBAction)stopSelect:(id)sender
+{
+  [selectProgress setShouldStop:YES];
 }
 
 /* INSERT */
 
 
+- (void)resetInsertUI:(id)arg
+{
+  [buttonInsertExec setEnabled:YES];
+  [buttonInsertStop setEnabled:NO];
+}
+
+
 - (IBAction)showInsert:(id)sender
 {
   NSArray *objectNames;
@@ -437,8 +474,7 @@
   NS_HANDLER
     if ([[localException name] hasPrefix:@"DB"])
       {
-        [faultTextView setString:[localException reason]];
-        [faultPanel makeKeyAndOrderFront:nil];
+        [self performSelectorOnMainThread:@selector(showException:) withObject:localException waitUntilDone:YES];
       }
   NS_ENDHANDLER
   [popupObjectsInsert removeAllItems];
@@ -462,20 +498,21 @@
     }
 }
 
-- (IBAction)executeInsert:(id)sender
+- (void)performInsert:(id)arg
 {
   NSString       *filePath;
   NSString       *resFilePath;
   DBCSVReader    *reader;
   NSString       *intoWhichObject;
-  DBProgress     *progress;
   NSMutableArray *results;
   NSFileManager  *fileManager;
   NSFileHandle   *resFH;
   NSUserDefaults *defaults;
   DBCSVWriter    *resWriter;
   NSString       *str;
+  NSAutoreleasePool *arp;
   
+  arp = [NSAutoreleasePool new];
   defaults = [NSUserDefaults standardUserDefaults];  
   filePath = [fieldFileInsert stringValue];
   resFilePath = [[filePath stringByDeletingLastPathComponent] stringByAppendingPathComponent: @"results.csv"];
@@ -487,11 +524,11 @@
   intoWhichObject = [[[popupObjectsInsert selectedItem] title] retain];
   [logger log:LogInformative :@"[AppController executeInsert] object: %@\n", intoWhichObject];
 
-  progress = [[DBProgress alloc] init];
-  [progress setLogger:logger];
-  [progress setProgressIndicator: progIndInsert];
-  [progress setRemainingTimeField: fieldRTInsert];
-  [progress reset];
+  insertProgress = [[DBProgress alloc] init];
+  [insertProgress setLogger:logger];
+  [insertProgress setProgressIndicator: progIndInsert];
+  [insertProgress setRemainingTimeField: fieldRTInsert];
+  [insertProgress reset];
   
   results = nil;
   reader = [[DBCSVReader alloc] initWithPath:filePath withLogger:logger];
@@ -504,13 +541,15 @@
   [reader parseHeaders];
 
   NS_DURING
-    results = [dbCsv create:intoWhichObject fromReader:reader progressMonitor:progress];
+    results = [dbCsv create:intoWhichObject fromReader:reader progressMonitor:insertProgress];
     [results retain];
   NS_HANDLER
     if ([[localException name] hasPrefix:@"DB"])
       {
-        [faultTextView setString:[localException reason]];
-        [faultPanel makeKeyAndOrderFront:nil];
+        [self performSelectorOnMainThread:@selector(showException:) withObject:localException waitUntilDone:YES];
+        [insertProgress release];
+        [self resetInsertUI:self];
+        [arp drain];
       }
   NS_ENDHANDLER
 
@@ -554,8 +593,23 @@
 
   [reader release];
   [intoWhichObject release];
-  [progress release];
+  [insertProgress release];
+  insertProgress = nil;
+  [self performSelectorOnMainThread:@selector(resetInsertUI:) withObject:self waitUntilDone:NO];
   [results release];
+  [arp drain];
+}
+
+- (IBAction)executeInsert:(id)sender
+{
+  [buttonInsertExec setEnabled:NO];
+  [buttonInsertStop setEnabled:YES];
+  [NSThread detachNewThreadSelector:@selector(performInsert:) toTarget:self withObject:nil];
+}
+
+- (IBAction)stopInsert:(id)sender
+{
+  [insertProgress setShouldStop:YES];
 }
 
 /* UPDATE */
@@ -573,8 +627,7 @@
   NS_HANDLER
     if ([[localException name] hasPrefix:@"DB"])
       {
-        [faultTextView setString:[localException reason]];
-        [faultPanel makeKeyAndOrderFront:nil];
+        [self performSelectorOnMainThread:@selector(showException:) withObject:localException waitUntilDone:YES];
       }
   NS_ENDHANDLER
   [popupObjectsUpdate removeAllItems];
@@ -598,31 +651,39 @@
     }
 }
 
-- (IBAction)executeUpdate:(id)sender
+- (void)resetUpdateUI:(id)arg
+{
+  [buttonUpdateExec setEnabled:YES];
+  [buttonUpdateStop setEnabled:NO];
+}
+
+- (void)performUpdate:(id)arg
 {
   NSString       *filePath;
   NSString       *resFilePath;
   DBCSVReader    *reader;
   NSString       *whichObject;
-  DBProgress     *progress;
   NSMutableArray *results;
   NSFileManager  *fileManager;
   NSFileHandle   *resFH;
   NSUserDefaults *defaults;
   DBCSVWriter    *resWriter;
   NSString       *str;
-
+  NSAutoreleasePool *arp;
+  
+  arp = [NSAutoreleasePool new];
+  
   defaults = [NSUserDefaults standardUserDefaults];  
   filePath = [fieldFileUpdate stringValue];
   resFilePath = [[filePath stringByDeletingLastPathComponent] stringByAppendingPathComponent: @"results.csv"];
 
   [logger log:LogDebug: @"[AppController executeUpdate] writing results to: %@", resFilePath];
   
-  progress = [[DBProgress alloc] init];
-  [progress setLogger:logger];
-  [progress setProgressIndicator: progIndUpdate];
-  [progress setRemainingTimeField: fieldRTUpdate];
-  [progress reset];
+  updateProgress = [[DBProgress alloc] init];
+  [updateProgress setLogger:logger];
+  [updateProgress setProgressIndicator: progIndUpdate];
+  [updateProgress setRemainingTimeField: fieldRTUpdate];
+  [updateProgress reset];
 
   whichObject = [[[popupObjectsUpdate selectedItem] title] retain];
   [logger log:LogInformative :@"[AppController executeUpdate] object: %@\n", whichObject];
@@ -638,13 +699,16 @@
   [reader parseHeaders];
 
   NS_DURING
-    results = [dbCsv update:whichObject fromReader:reader progressMonitor:progress];
+    results = [dbCsv update:whichObject fromReader:reader progressMonitor:updateProgress];
     [results retain];
   NS_HANDLER
     if ([[localException name] hasPrefix:@"DB"])
       {
-        [faultTextView setString:[localException reason]];
-        [faultPanel makeKeyAndOrderFront:nil];
+        [self performSelectorOnMainThread:@selector(showException:) withObject:localException waitUntilDone:YES];
+      }
+    else
+      {
+        [localException raise];
       }
   NS_ENDHANDLER
 
@@ -687,8 +751,24 @@
     
   [reader release];
   [whichObject release];
-  [progress release];
+  [updateProgress release];
+  updateProgress = nil;
   [results release];
+  [self performSelectorOnMainThread:@selector(resetUpdateUI:) withObject:self waitUntilDone:NO];
+  [arp drain];
+}
+
+- (IBAction)executeUpdate:(id)sender
+{
+  [buttonUpdateExec setEnabled:NO];
+  [buttonUpdateStop setEnabled:YES];
+  [NSThread detachNewThreadSelector:@selector(performUpdate:) toTarget:self withObject:nil];
+}
+
+
+- (IBAction)stopUpdate:(id)sender
+{
+  [updateProgress setShouldStop:YES];
 }
 
 /*  SELECT IDENTIFY */
@@ -729,7 +809,13 @@
     }
 }
 
-- (IBAction)executeSelectIdentify:(id)sender
+- (void)resetSelectIdentUI:(id)arg
+{
+  [buttonSelectIdentExec setEnabled:YES];
+  [buttonSelectIdentStop setEnabled:NO];
+}
+
+- (void)performSelectIdentify:(id)arg
 {
   NSString       *statement;
   NSString       *filePathIn;
@@ -738,11 +824,12 @@
   NSFileManager  *fileManager;
   DBCSVWriter    *csvWriter;
   DBCSVReader    *csvReader;
-  DBProgress     *progress;
   int            batchSize;
   NSString       *str;
   NSUserDefaults *defaults;
+  NSAutoreleasePool *arp;
 
+  arp = [NSAutoreleasePool new];
   defaults = [NSUserDefaults standardUserDefaults];
 
   statement = [fieldQuerySelectIdentify string];
@@ -761,6 +848,12 @@
     case 3:
       batchSize = 50;
       break;
+    case 4:
+      batchSize = 100;
+      break;      
+    case 5:
+      batchSize = 200;
+      break;      
     case 99:
       batchSize = -1;
       break;
@@ -783,6 +876,8 @@
     {
       NSRunAlertPanel(@"Attention", @"Could not create File.", @"Ok", nil, nil);
       [csvReader release];
+      [arp drain];
+      [self performSelectorOnMainThread:@selector(resetSelectIdentUI:) withObject:self waitUntilDone:NO];
       return;
     }  
 
@@ -791,6 +886,8 @@
     {
       NSRunAlertPanel(@"Attention", @"Cannot create File.", @"Ok", nil, nil);
       [csvReader release];
+      [arp drain];
+      [self performSelectorOnMainThread:@selector(resetSelectIdentUI:) withObject:self waitUntilDone:NO];
       return;
     }
 
@@ -803,20 +900,19 @@
   str = [defaults stringForKey:@"CSVWriteSeparator"];
   if (str)
     [csvWriter setSeparator:str];
-
-  progress = [[DBProgress alloc] init];
-  [progress setLogger:logger];
-  [progress setProgressIndicator: progIndSelectIdent];
-  [progress setRemainingTimeField: fieldRTSelectIdent];
-  [progress reset];
+  [csvWriter setLineBreakHandling:[defaults integerForKey:CSVWriteLineBreakHandling]];
+  selectIdentProgress = [[DBProgress alloc] init];
+  [selectIdentProgress setLogger:logger];
+  [selectIdentProgress setProgressIndicator: progIndSelectIdent];
+  [selectIdentProgress setRemainingTimeField: fieldRTSelectIdent];
+  [selectIdentProgress reset];
 
   NS_DURING
-    [dbCsv queryIdentify :statement queryAll:([queryAllSelectIdentify state] == NSOnState) fromReader:csvReader toWriter:csvWriter withBatchSize:batchSize progressMonitor:progress];
+    [dbCsv queryIdentify :statement queryAll:([queryAllSelectIdentify state] == NSOnState) fromReader:csvReader toWriter:csvWriter withBatchSize:batchSize progressMonitor:selectIdentProgress];
   NS_HANDLER
     if ([[localException name] hasPrefix:@"DB"])
       {
-        [faultTextView setString:[localException reason]];
-        [faultPanel makeKeyAndOrderFront:nil];
+        [self performSelectorOnMainThread:@selector(showException:) withObject:localException waitUntilDone:YES];
       }
   NS_ENDHANDLER
 
@@ -824,7 +920,23 @@
   [csvWriter release];
   [fileHandleOut closeFile];
   
-  [progress release];
+  [selectIdentProgress release];
+  selectIdentProgress = nil;
+  [self performSelectorOnMainThread:@selector(resetSelectIdentUI:) withObject:self waitUntilDone:NO];
+  [arp drain];
+}
+
+- (IBAction)executeSelectIdentify:(id)sender
+{
+  [buttonSelectIdentExec setEnabled:NO];
+  [buttonSelectIdentStop setEnabled:YES];
+  [NSThread detachNewThreadSelector:@selector(performSelectIdentify:) toTarget:self withObject:nil];
+}
+
+
+- (IBAction)stopSelectIdentify:(id)sender
+{
+  [selectIdentProgress setShouldStop:YES];
 }
 
 /* DESCRIBE */
@@ -902,8 +1014,7 @@
   NS_HANDLER
     if ([[localException name] hasPrefix:@"DB"])
       {
-        [faultTextView setString:[localException reason]];
-        [faultPanel makeKeyAndOrderFront:nil];
+        [self performSelectorOnMainThread:@selector(showException:) withObject:localException waitUntilDone:YES];
       }
   NS_ENDHANDLER
     
@@ -919,30 +1030,46 @@
   [winQuickDelete makeKeyAndOrderFront:self];
 }
 
+- (void)resetQuickDeleteUI:(id)arg
+{
+  [buttonQuickDeleteExec setEnabled:YES];
+}
 
-- (IBAction)quickDelete:(id)sender
+
+- (void)performQuickDelete:(id)sender
 {
   NSString  *objectId;
   NSArray   *idArray;
   NSMutableArray *resultArray;
-
+  NSAutoreleasePool *arp;
+  GWSService     *serv;
+  DBSoap         *dbSoap;
+  
+  arp = [NSAutoreleasePool new];
+  
   resultArray = nil;
   [fieldStatusQd setStringValue:@""];
   objectId = [fieldObjectIdQd stringValue];
   
   if (objectId == nil || [objectId length] == 0)
     return;
+
+  /* we clone the soap instance and pass the session, so that the method can run in a separate thread */
+  dbSoap = [[DBSoap alloc] init];
+  serv = [DBSoap gwserviceForDBSoap];
+  [dbSoap setSessionId:[db sessionId]];
+  [serv setURL:[db serverUrl]];  
+  [dbSoap setService:serv];
   
   idArray = [NSArray arrayWithObject:objectId];
   
   NS_DURING
     [fieldStatusQd setStringValue:@"Working..."];
-    resultArray = [db delete: idArray progressMonitor:nil];
+    resultArray = [dbSoap delete: idArray progressMonitor:nil];
   NS_HANDLER
     if ([[localException name] hasPrefix:@"DB"])
       {
-        [faultTextView setString:[localException reason]];
-        [faultPanel makeKeyAndOrderFront:nil];
+        [self performSelectorOnMainThread:@selector(showException:) withObject:localException waitUntilDone:YES];
       }
   NS_ENDHANDLER
 
@@ -973,6 +1100,14 @@
         [faultTextView setString:resultMsgStr];
         [faultPanel makeKeyAndOrderFront:nil];
       }
+  [self performSelectorOnMainThread:@selector(resetQuickDeleteUI:) withObject:self waitUntilDone:NO];    
+  [arp drain];
+}
+
+- (IBAction)quickDelete:(id)sender
+{
+  [buttonQuickDeleteExec setEnabled:NO];
+  [NSThread detachNewThreadSelector:@selector(performQuickDelete:) toTarget:self withObject:nil];
 }
 
 /* DELETE */
@@ -999,7 +1134,13 @@
     }
 }
 
-- (IBAction)executeDelete:(id)sender
+- (void)resetDeleteUI:(id)arg
+{
+  [buttonDeleteExec setEnabled:YES];
+  [buttonDeleteStop setEnabled:NO];
+}
+
+- (void)performDelete:(id)arg
 {
   NSString       *filePath;
   NSString       *resFilePath;
@@ -1009,9 +1150,10 @@
   NSFileManager  *fileManager;
   NSFileHandle   *resFH;
   NSUserDefaults *defaults;
-  DBProgress     *progress;
   NSString       *str;
+  NSAutoreleasePool *arp;
 
+  arp = [NSAutoreleasePool new];
   defaults = [NSUserDefaults standardUserDefaults];  
   filePath = [fieldFileDelete stringValue];
   resFilePath = [[filePath stringByDeletingLastPathComponent] stringByAppendingPathComponent: @"results.csv"];
@@ -1027,21 +1169,20 @@
     [reader setSeparator:str];
   /* no need to reparse the headers since they are not used, just skipped */
 
-  progress = [[DBProgress alloc] init];
-  [progress setProgressIndicator: progIndDelete];
-  [progress setRemainingTimeField: fieldRTDelete];
-  [progress setLogger:logger];
-  [progress reset];
+  deleteProgress = [[DBProgress alloc] init];
+  [deleteProgress setProgressIndicator: progIndDelete];
+  [deleteProgress setRemainingTimeField: fieldRTDelete];
+  [deleteProgress setLogger:logger];
+  [deleteProgress reset];
 
   results = nil;  
   NS_DURING
-    results = [dbCsv deleteFromReader:reader progressMonitor:progress];
+    results = [dbCsv deleteFromReader:reader progressMonitor:deleteProgress];
     [results retain];
   NS_HANDLER
     if ([[localException name] hasPrefix:@"DB"])
       {
-        [faultTextView setString:[localException reason]];
-        [faultPanel makeKeyAndOrderFront:nil];
+        [self performSelectorOnMainThread:@selector(showException:) withObject:localException waitUntilDone:YES];
       }
   NS_ENDHANDLER
 
@@ -1075,7 +1216,22 @@
     }
   [results release];
   [reader release];
-  [progress release];
+  [deleteProgress release];
+  deleteProgress = nil;
+  [self performSelectorOnMainThread:@selector(resetDeleteUI:) withObject:self waitUntilDone:NO];
+  [arp drain];
+}
+
+- (IBAction)executeDelete:(id)sender
+{
+  [buttonDeleteExec setEnabled:NO];
+  [buttonDeleteStop setEnabled:YES];
+  [NSThread detachNewThreadSelector:@selector(performDelete:) toTarget:self withObject:nil];
+}
+
+- (IBAction)stopDelete:(id)sender
+{
+  [deleteProgress setShouldStop:YES];
 }
 
 /* OBJECT INSPECTOR */
