@@ -40,6 +40,7 @@
   unsigned long  maxVal;
   float          percent;
   NSDate         *startDate;
+  BOOL           shouldStop;
 
   NSProgressIndicator *progInd;
   NSTextField *fieldRemainingTime;
@@ -52,6 +53,9 @@
 - (void)setProgressIndicator:(NSProgressIndicator *)indicator;
 - (void)setRemainingTimeField:(NSTextField *)field;
 
+- (BOOL)shouldStop;
+- (void)setShouldStop:(BOOL)flag;
+
 @end
 
 
