@@ -1,7 +1,7 @@
 /*
   Project: DataBasin
 
-  Copyright (C) 2013-2014 Free Software Foundation
+  Copyright (C) 2013-2015 Free Software Foundation
   
   Author: Riccardo Mottola
   
@@ -28,6 +28,8 @@
 #import "Preferences.h"
 #import "AppController.h"
 
+#import <DataBasinKit/DBCSVWriter.h>
+
 #if defined(__APPLE__) && (MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_4)
 #define NSUTF16StringEncoding 999
 #endif
@@ -58,11 +60,14 @@
 #endif
   
   bCell = [[NSButtonCell alloc] init];
+#if defined(__APPLE__)
+  [bCell setBezelStyle:NSShadowlessSquareBezelStyle];
+#endif
   buttonMatrix = [[NSMatrix alloc] initWithFrame:NSZeroRect mode:NSRadioModeMatrix prototype:bCell numberOfRows:1 numberOfColumns:3];
   [buttonMatrix setTarget: self];
   [buttonMatrix setAction:@selector(changePrefView:)];
   [buttonMatrix setAllowsEmptySelection:NO];
-  [buttonMatrix setCellSize: NSMakeSize(80, 40)];
+  [buttonMatrix setCellSize: NSMakeSize(85, 40)];
   [matrixScrollView setDocumentView:buttonMatrix];
   [bCell release];
 
@@ -148,6 +153,20 @@
   if (value)
     [fieldWriteSeparator setStringValue:value];
 
+  
+  i = [defaults integerForKey:CSVWriteLineBreakHandling];
+  switch (i)
+    {
+    case DBCSVLineBreakDelete:
+      [matrixWriteLineBreak selectCellAtRow:1  column:0];
+      break;
+    case DBCSVLineBreakReplaceWithSpace:
+      [matrixWriteLineBreak selectCellAtRow:2  column:0];
+      break;
+    case DBCSVLineBreakNoChange:
+    default: /* default treated as DBCSVLineBreakNoChange */
+      [matrixWriteLineBreak selectCellAtRow:0  column:0];
+    }
   [buttonMatrix selectCellAtRow:0 column:0];
   [buttonMatrix sendAction];
   [prefPanel makeKeyAndOrderFront:self];
@@ -250,6 +269,24 @@
       return;
     }
 
+  switch ([matrixWriteLineBreak selectedRow])
+    {
+    case -1:
+      NSLog(@"unselected value in matrixWriteLineBreak selection");
+      break;
+    case 0:
+      [defaults setObject:[NSNumber numberWithInt:DBCSVLineBreakNoChange] forKey:CSVWriteLineBreakHandling];
+      break;
+    case 1:
+        [defaults setObject:[NSNumber numberWithInt:DBCSVLineBreakDelete] forKey:CSVWriteLineBreakHandling];
+      break;
+    case 2:
+      [defaults setObject:[NSNumber numberWithInt:DBCSVLineBreakReplaceWithSpace] forKey:CSVWriteLineBreakHandling];
+      break;
+    default:
+      NSLog(@"unexpected value in matrixWriteLineBreak selection");
+    }
+
   [prefPanel performClose: nil];
 
   [appController reloadDefaults];
